# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Tests for Shenzhen dataset."""

import importlib

import pytest


def test_protocol_consistency():
    # Default protocol

    datamodule = getattr(
        importlib.import_module("ptbench.data.shenzhen.datamodules"), "default"
    )

    subset = datamodule.splits

    assert len(subset) == 3

    assert "train" in subset
    train_samples = subset["train"][0][0]
    assert len(train_samples) == 422
    for s in train_samples:
        assert s[0].startswith("CXR_png/CHNCXR_0")

    assert "validation" in subset
    validation_samples = subset["validation"][0][0]
    assert len(validation_samples) == 107
    for s in validation_samples:
        assert s[0].startswith("CXR_png/CHNCXR_0")

    assert "test" in subset
    test_samples = subset["test"][0][0]
    assert len(test_samples) == 133
    for s in test_samples:
        assert s[0].startswith("CXR_png/CHNCXR_0")

    # Check labels
    for s in train_samples:
        assert s[1] in [0.0, 1.0]

    for s in validation_samples:
        assert s[1] in [0.0, 1.0]

    for s in test_samples:
        assert s[1] in [0.0, 1.0]

    # Cross-validation folds 0-1
    for f in range(2):
        datamodule = getattr(
            importlib.import_module("ptbench.data.shenzhen.datamodules"),
            f"fold_{str(f)}",
        )

        subset = datamodule.splits

        assert len(subset) == 3

        assert "train" in subset
        train_samples = subset["train"][0][0]
        assert len(train_samples) == 476
        for s in train_samples:
            assert s[0].startswith("CXR_png/CHNCXR_0")

        assert "validation" in subset
        validation_samples = subset["validation"][0][0]
        assert len(validation_samples) == 119
        for s in validation_samples:
            assert s[0].startswith("CXR_png/CHNCXR_0")

        assert "test" in subset
        test_samples = subset["test"][0][0]
        assert len(test_samples) == 67
        for s in test_samples:
            assert s[0].startswith("CXR_png/CHNCXR_0")

        # Check labels
        for s in train_samples:
            assert s[1] in [0.0, 1.0]

        for s in validation_samples:
            assert s[1] in [0.0, 1.0]

        for s in test_samples:
            assert s[1] in [0.0, 1.0]

    # Cross-validation folds 2-9
    for f in range(2, 10):
        datamodule = getattr(
            importlib.import_module("ptbench.data.shenzhen.datamodules"),
            f"fold_{str(f)}",
        )

        subset = datamodule.splits

        assert len(subset) == 3

        assert "train" in subset
        train_samples = subset["train"][0][0]
        assert len(train_samples) == 476
        for s in train_samples:
            assert s[0].startswith("CXR_png/CHNCXR_0")

        assert "validation" in subset
        validation_samples = subset["validation"][0][0]
        assert len(validation_samples) == 120
        for s in validation_samples:
            assert s[0].startswith("CXR_png/CHNCXR_0")

        assert "test" in subset
        test_samples = subset["test"][0][0]
        assert len(test_samples) == 66
        for s in test_samples:
            assert s[0].startswith("CXR_png/CHNCXR_0")

        # Check labels
        for s in train_samples:
            assert s[1] in [0.0, 1.0]

        for s in validation_samples:
            assert s[1] in [0.0, 1.0]

        for s in test_samples:
            assert s[1] in [0.0, 1.0]


@pytest.mark.skip_if_rc_var_not_set("datadir.shenzhen")
def test_loading():
    import torch
    import torchvision.transforms

    from ptbench.data.datamodule import _DelayedLoadingDataset

    def _check_sample(s):
        assert len(s) == 2

        data = s[0]
        metadata = s[1]

        assert isinstance(data, torch.Tensor)

        assert data.size(0) == 1  # check 1 channel
        assert data.size(1) == data.size(2)  # check square image

        assert (
            torchvision.transforms.ToPILImage()(data).mode == "L"
        )  # Check colors

        assert "label" in metadata
        assert metadata["label"] in [0, 1]  # Check labels

    limit = 30  # use this to limit testing to first images only, else None

    module = importlib.import_module("ptbench.data.shenzhen.datamodules")
    datamodule = getattr(module, "default")
    raw_data_loader = module.RawDataLoader()
    subset = datamodule.splits

    # Need to use private function so we can limit the number of samples to use
    dataset = _DelayedLoadingDataset(
        subset["train"][0][0][:limit],
        raw_data_loader,
    )

    for s in dataset:
        _check_sample(s)


@pytest.mark.skip(reason="Test need to be updated")
@pytest.mark.skip_if_rc_var_not_set("datadir.shenzhen")
def test_check():
    from ptbench.data.split import check_database_split_loading

    limit = 30  # use this to limit testing to first images only, else 0

    # Default protocol
    module = importlib.import_module("ptbench.data.shenzhen.datamodules")
    datamodule = getattr(module, "default")
    database_split = datamodule.splits
    raw_data_loader = module.RawDataLoader()

    assert (
        check_database_split_loading(
            database_split, raw_data_loader, limit=limit
        )
        == 0
    )

    # Folds
    for f in range(10):
        module = importlib.import_module("ptbench.data.shenzhen.datamodules")
        datamodule = getattr(module, f"fold_{f}")

        database_split = datamodule.splits
        raw_data_loader = module.RawDataLoader()

        assert (
            check_database_split_loading(
                database_split, raw_data_loader, limit=limit
            )
            == 0
        )
